@@ -1,4 +1,5 @@
 import dataclasses as dc
+import json
 import logging
 from abc import ABC, abstractmethod
 from collections.abc import Callable
@@ -27,7 +28,7 @@ def register_message_type (msg_type: str) -> Callable:
 
     :param msg_type: Message type to be registered
     :return: Same object
-    """    
+    """
     def _decorator (cls: RECEIVED_MESSAGE_T) -> RECEIVED_MESSAGE_T:
         RECEIVED_MSG_TYPE_TO_OBJECT[msg_type] = cls
         return cls
@@ -113,7 +114,7 @@ class Context:
 @dc.dataclass
 class Metadata:
     display_phone_number: str = dc.field()
-    "Phone number that cusomer will see in chat"
+    "Phone number that customer will see in chat"
     phone_number_id: str = dc.field()
     "Phone number id. Need to be used to respond an message "
 
@@ -215,11 +216,11 @@ class Statuses:
     "Customer's WhatsApp ID. Can be used to respond a customer (Doesn't match with phone number)"
     biz_opaque_callback_data: str | None = dc.field(default=None)
     "Arbitrary string"
-    conversation: Conversation = dc.field(default=None)
+    conversation: Conversation | None = dc.field(default=None)
     "Conversation info"
     errors: list[Errors] = dc.field(default_factory=list)
     "List of objects describing errors"
-    pricing: Pricing = dc.field(default=None)
+    pricing: Pricing | None = dc.field(default=None)
     "Pricing information"
 
 @dataclass_json
@@ -328,7 +329,7 @@ class AudioMessage (ReceivedMessage):
     """
     Send and receive audios.
 
-    Supoorted audio formats: aac, amr, mp3, mp4 audio, ogg (opus codecs, not audio/ogg)  
+    Supoorted audio formats: aac, amr, mp3, mp4 audio, ogg (opus codecs, not audio/ogg)
     Max size: 16 MB
     """
     audio: DocumentMetadata = dc.field(kw_only=True)
@@ -378,7 +379,7 @@ class DocumentMessage (ReceivedMessage):
     """
     Message that display a icon and can be downloaded by user when tapped
 
-    Max Size: 100 MB  
+    Max Size: 100 MB
     Supported MIMEs (message custom logo):
     - text/plain
     - application/vnd.ms-excel
@@ -465,8 +466,38 @@ class ImageMessage (ReceivedMessage):
     def image_id (self, value: str):
         raise ValueError("Can't set 'image_id' directly. Use 'image.id' instead.")
 
-class ButtonUrlMessage (ReceivedMessage):
-    ...
+@dataclass_json
+@dc.dataclass
+class ButtonUrlMessage:
+    header: str = dc.field(kw_only=True)
+    "Header text"
+    body: str = dc.field(kw_only=True)
+    "Body Text"
+    footer: str = dc.field(kw_only=True)
+    "Message footer text"
+
+    button_display_text: str = dc.field(kw_only=True)
+    "Button text"
+    button_url: str = dc.field(kw_only=True)
+    "URL that browser will open when user tap the button"
+
+    def to_send (self, to: str) -> dict[str, str]:
+        return {
+            **ReceivedMessage.default_body_to_send(to, MessageTypes.INTERACTIVE),
+            "interactive": {
+                "type": "cta_url",
+                "header": { "type": "text", "text": self.header },
+                "body": { "text": self.body },
+                "footer": { "text": self.footer },
+                "action": {
+                    "name": "cta_url",
+                    "parameters": {
+                        "display_text": self.button_display_text,
+                        "url": self.button_url
+                    }
+                }
+            }
+        }
 
 class FlowMessage (ReceivedMessage):
     # TODO Implement FlowMessage when I receive API access
@@ -478,8 +509,8 @@ class InteractiveListMessage:
     header: str = dc.field(kw_only=True)
     body: str = dc.field(kw_only=True)
     footer: str = dc.field(kw_only=True)
-    button_title: str = dc.field(kw_only=True)
 
+    button_title: str = dc.field(kw_only=True)
     sections: list[Section] = dc.field(kw_only=True)
 
     def to_send (self, to: str) -> dict[str, str]:
@@ -509,7 +540,13 @@ class InteractiveListReply (ReceivedMessage):
 
     @property
     def list_reply(self) -> Item:
-        return Item.from_dict(self.interactive["list_reply"])
+        item = self.interactive["list_reply"]
+
+        if isinstance(item, str):
+            item = item.replace("'", "\"")
+            item = json.loads(item)
+
+        return Item.from_dict(item)
 
     @list_reply.setter
     def list_reply(self, value: Item):
@@ -596,8 +633,22 @@ class LocationMessage (ReceivedMessage):
 
 @dataclass_json
 @dc.dataclass
-class AskForLocationMessage (ReceivedMessage):
-    ...
+class AskForLocationMessage:
+    """
+    Max body text size: 4096 characters
+    """
+    body: str = dc.field(kw_only=True)
+    "Text body of location request (Accepts URL link). Max 4096 characters"
+
+    def to_send (self, to: str) -> dict[str, str]:
+        return {
+            **ReceivedMessage.default_body_to_send(to, MessageTypes.INTERACTIVE),
+            "interactive": {
+                "type": "location_request_message",
+                "body": { "text": self.body },
+                "action": { "name": "send_location" }
+            }
+        }
 
 @dataclass_json
 @dc.dataclass
@@ -652,9 +703,9 @@ class StickerMessage (ReceivedMessage):
 @register_message_type(MessageTypes.TEXT)
 class TextMessage (ReceivedMessage):
     """
-    Text message. Contains information about sent and received texts.  
+    Text message. Contains information about sent and received texts.
     Shows a website preview if message has a url (startswith http:// or https://)
-    and preview_url is True, the url will be previewed.  
+    and preview_url is True, the url will be previewed.
     Max text size: 4096 characters
     """
     text: dict[str, str] = dc.field(kw_only=True)
@@ -673,7 +724,7 @@ class TextMessage (ReceivedMessage):
         :param message: Text message to send, max 4096 characters
         :param preview_url: Show website preview if has an url (http/s) and is True, defaults to None
         :return: Dictionary representing the reply message
-        """        
+        """
         output_msg = cls.default_body_to_send(to, MessageTypes.TEXT)
         output_msg["text"] = { "body": message }
 
@@ -688,8 +739,8 @@ class TextMessage (ReceivedMessage):
 class VideoMessage (ReceivedMessage):
     """
     Video message with an optinal caption.
-    
-    Supoorted audio formats: 3gp and mp4  
+
+    Supoorted audio formats: 3gp and mp4
     Max size: 16 MB
     """
     video: DocumentMetadata = dc.field(kw_only=True)
@@ -739,7 +790,7 @@ def map_received_msg_to_obj (messages: list[dict[str, Any]] | None) -> list[RECE
     :param messages: List of received messages as dictionaries.
     :raises NotImplementedMsgType: If message type is not supported.
     :return: List of converted messages.
-    """    
+    """
     if messages is None:
         return []
 
